import java.awt.*;
import java.awt.image.*;
class Main extends Frame {
    Main(String filename) {
        super("CropImage Example");
        try {
            Image fileImage = getToolkit().getImage(filename);

            // Use a media tracker object to wait until all the pixels 
            // have been retrieved.
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(fileImage, 0);
            tracker.waitForID(0);

            // Now copy the retrieved image to an offscreen image.
            addNotify();    // otherwise the following code will fail.
            Image image = createImage(fileImage.getWidth(this), 
                                      fileImage.getHeight(this));
            Graphics g = image.getGraphics();
            g.drawImage(fileImage, 0, 0, this);
            g.dispose();

            add("Center", new ImageCanvas(image));
        } catch (Exception e) {
            e.printStackTrace();
        }
        resize(300, 300);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    Image image;
    Rectangle selected = new Rectangle();
    Image cropImage;
    Rectangle cropImageRect = new Rectangle();

    ImageCanvas(Image image) {
        this.image = image;
    }

    public void update(Graphics g) {
        paint(g);
    }
    public void paint(Graphics g) {
        g.drawImage(image, 0, 0, this);
        if (cropImage != null) {
            g.drawImage(cropImage, cropImageRect.x, cropImageRect.y, this);
        } else if (!selected.isEmpty()) {
            g.setXORMode(getBackground());
            g.fillRect(selected.x, selected.y, 
                       selected.width, selected.height);
            g.setPaintMode();
        }
    }

    // Convenience method.
    void repaintRect(Rectangle r) {
        repaint(r.x, r.y, r.width, r.height);
    }
        
    public boolean mouseDown(Event evt, int x, int y) {
        if (cropImage != null) {
            // Add cropImage to main image.
            Graphics g = image.getGraphics();
            g.drawImage(cropImage, cropImageRect.x, cropImageRect.y, this);
            g.dispose();
            cropImage = null;
        } 
        repaintRect(selected);
        selected.move(x, y);
        repaintRect(selected);
        return true;
    }
    public boolean mouseDrag(Event evt, int x, int y) {
        repaintRect(selected);
        selected.width = Math.max(0, x - selected.x);
        selected.height = Math.max(0, y - selected.y);
        repaintRect(selected);
        return true;
    }
    public boolean mouseUp(Event evt, int x, int y) {
        mouseDrag(evt, x, y);
        return true;
    }
    public boolean mouseMove(Event evt, int x, int y) {
        if (cropImage != null) {
            // Move the cropped image around.
            repaintRect(cropImageRect);
            cropImageRect.move(x, y);
            repaintRect(cropImageRect);
            return true;
        }
        return false;
    }
    public boolean keyDown(Event evt, int key) {
        if (key == 'c') {
            if (!selected.isEmpty()) {
                // Create crop image using a CropImageFilter.
                CropImageFilter imgf = new CropImageFilter(
                    selected.x, selected.y, selected.width, selected.height);
                ImageProducer ip = image.getSource();
                ip = new FilteredImageSource(ip, imgf);
                cropImage = getToolkit().createImage(ip);
                cropImageRect = new Rectangle(selected.x, selected.y, 
                    selected.width, selected.height);
            }
        } else if (cropImage != null) {
            repaintRect(cropImageRect);
            cropImage = null;
        }
        // Delete the selection.
        repaintRect(selected);
        selected.width = selected.height = 0;
        return true;
    }
}
