import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.util.*;
class Main extends Frame {
    Main(String filename) {
        super("ImageObserver Example");
        try {
            // Retrieve the image.
            Image image = getToolkit().getImage(filename);

            add("Center", new ImageCanvas(image));
        } catch (Exception e) {
            e.printStackTrace();
        }
        resize(50, 100);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    Image image;
    ImageObserverSpy spy = new ImageObserverSpy(this);

    ImageCanvas(Image image) {
        this.image = image;
        image.getWidth(spy);
        image.getHeight(spy);
        image.getProperty("test", spy);
    }

    public void paint(Graphics g) {
        g.drawImage(image, 0, 0, spy);
    }
}

class ImageObserverSpy implements ImageObserver {
    ImageObserver obs;

    ImageObserverSpy(ImageObserver obs) {
        this.obs = obs;
    }
    
    public boolean imageUpdate(Image img, int infoflags,
        int x, int y, int width, int height) {
        System.out.print("x="+x+" ");
        System.out.print("y="+y+" ");
        System.out.print("width="+width+" ");
        System.out.print("height="+height+" ");
        System.out.print("  infoflags=");
        if ((infoflags & ABORT) != 0) System.out.print("ABORT ");
        if ((infoflags & ALLBITS) != 0) System.out.print("ALLBITS ");
        if ((infoflags & ERROR) != 0) System.out.print("ERROR ");
        if ((infoflags & FRAMEBITS) != 0) System.out.print("FRAMEBITS ");
        if ((infoflags & HEIGHT) != 0) System.out.print("HEIGHT ");
        if ((infoflags & PROPERTIES) != 0) System.out.print("PROPERTIES ");
        if ((infoflags & SOMEBITS) != 0) System.out.print("SOMEBITS ");
        if ((infoflags & WIDTH)!= 0) System.out.print("WIDTH ");
        System.out.println();
        return obs.imageUpdate(img, infoflags, x, y, width, height);
    }
}
