import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.util.*;
class Main extends Frame {
    Main(String filename) {
        super("filterRGBPixels Example");
        try {
            add("Center", new ImageCanvas(getToolkit().getImage(filename)));
        } catch (Exception e) {
            e.printStackTrace();
        }
        resize(50, 100);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Canvas {
    Image gradientImage;

    ImageCanvas(Image image) {
        GradientFilter imgf = new GradientFilter();
        ImageProducer ip = image.getSource();
        ip = new FilteredImageSource(ip, imgf);
        gradientImage = getToolkit().createImage(ip);
        repaint();
    }

    public void paint(Graphics g) {
        g.drawImage(gradientImage, 0, 0, this);
    }
}

class GradientFilter extends RGBImageFilter {
    float[] hsb = new float[3];
    int width, height;

    public void setDimensions(int w, int h) {
        super.setDimensions(w, h);
        width = w;
        height = h;
    }

    public void filterRGBPixels(int x, int y, int w, int h,
                                int pixels[], int offset, int scansize) {
        int i = offset;
        for (int cy = 0; cy < h; cy++) {
            for (int cx = 0; cx < w; cx++) {
                Color c = new Color(pixels[i]);

                Color.RGBtoHSB(c.getRed(), c.getGreen(), c.getBlue(), hsb);
                hsb[2] += .5f - (float)x / width;
                hsb[2] += .5f - (float)y / height;
                hsb[2] = Math.max(0.0f, Math.min(1.0f, hsb[2]));
                pixels[i] = Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]);
                i++;
            }
            i += scansize - w;
        }
        consumer.setPixels(x, y, w, h, ColorModel.getRGBdefault(),
                           pixels, offset, scansize);
    }

    // Never called.
    public int filterRGB(int x, int y, int rgb) {
        return 0;
    }
}

