import java.awt.*;
class Main extends Frame {
    Main() {
        super("Container example");
        MenuBar mb = new MenuBar();

        // Edit Menu 
        Menu m = new Menu("Edit");
        m.add("Remove All");
        m.add(new CheckboxMenuItem("Test"));
        mb.add(m);

        // Add Menu 
        m = new Menu("Add");
        m.add("Button");
        m.add("Checkbox");
        m.add("Choice");
        m.add("List");
        m.add("Scrollbar");
        m.add("TextArea");
        m.add("TextField");
        mb.add(m);

        setMenuBar(mb);

        // Remove the default layout manager.
        setLayout(null);

        resize(400, 200);
        show();
    }

    Component dragging;
    Component stretching;
    Point offset;
    public boolean mouseDown(Event evt, int x, int y) {
        x += insets().left;
        y += insets().top;
        Component c = locate(x, y);
        if (c != null && c != this) {
            if (evt.shiftDown() && evt.controlDown()) {
                remove(c);
            } else if (evt.metaDown()) {
                stretching = c;
            } else {
                offset = new Point(x-c.location().x, y-c.location().y);
                dragging = c;
            }
        }
        return true;
    }

    public boolean mouseDrag(Event evt, int x, int y) {
        x += insets().left;
        y += insets().top;
        if (dragging != null) {
            dragging.move(x-offset.x, y-offset.y);
        } else if (stretching != null) {
            stretching.resize(x-stretching.bounds().x, 
                              y-stretching.bounds().y);
        }
        return true;
    }

    public boolean mouseUp(Event evt, int x, int y) {
        stretching = dragging = null;
        return true;
    }

    int newX, newY;
    public boolean action(Event evt, Object arg) {
        newX = Math.max(insets().left, newX);
        newY = Math.max(insets().top, newY);
        Component c = null;

        if (evt.target instanceof MenuItem) {
            if ("Button".equals(arg)) {
                add(c = new Button("Button"));
            } else if ("Checkbox".equals(arg)) {
                add(c = new Checkbox("Checkbox"));
            } else if ("Choice".equals(arg)) {
                add(c = new Choice());
            } else if ("List".equals(arg)) {
                add(c = new List());
            } else if ("Scrollbar".equals(arg)) {
                add(c = new Scrollbar());
            } else if ("TextArea".equals(arg)) {
                add(c = new TextArea("TextArea", 3, 20));
            } else if ("TextField".equals(arg)) {
                add(c = new TextField("TextField"));
            } else if ("Remove All".equals(arg)) {
                removeAll();
            } else if ("Test".equals(arg)) {
                Component[] all = getComponents();

                for (int i=0; i<all.length; i++) {
                    all[i].enable(((CheckboxMenuItem)evt.target).getState());
                }
            }
        }
        if (c != null) {
            Dimension d = c.preferredSize();

            c.reshape(newX, newY, d.width, d.height);
            c.disable();
            newX += 20;
            newY += 20;
            if (newX > size().width*3/4) {
                newX = 0;
            }
            if (newY > size().height*3/4) {
                newY = 0;
            }
            return true;
        }
        return false;
    }

    static public void main(String[] args) {
        new Main();
    }
}
