import java.awt.*;
import java.io.*;
import java.util.*;

class Main extends Frame {
    Vector tokens = new Vector();
    Font defaultFont = new Font("TimesRoman", Font.PLAIN, 12);

    Main(String filename) throws IOException {
        super("FontMetrics Example");
        StreamTokenizer st = 
            new StreamTokenizer(new FileInputStream(filename));
        Font f = defaultFont;;

        int t = st.nextToken();
        do {
            switch (t) {
            case '<':
                t = st.nextToken();
                switch (t) {
                case StreamTokenizer.TT_NUMBER:
                    f = new Font(f.getName(), f.getStyle(), (int)st.nval);
                    break;
                case StreamTokenizer.TT_WORD:
                    switch (st.sval.charAt(0)) {
                    case 'b':
                        f = new Font(f.getName(), Font.BOLD, f.getSize());
                        break;
                    case 'i':
                        f = new Font(f.getName(), Font.ITALIC, f.getSize());
                        break;
                    case 'p':
                        f = new Font(f.getName(), Font.PLAIN, f.getSize());
                        break;
                    default:
                        f = new Font(st.sval, f.getStyle(), f.getSize());
                        break;
                    }
                }
                tokens.addElement(f);
                break;
            case StreamTokenizer.TT_WORD:
                tokens.addElement(st.sval);
                break;
            case StreamTokenizer.TT_NUMBER:
                tokens.addElement(String.valueOf((int)st.nval));
                break;
            case StreamTokenizer.TT_EOL:
                break;
            default:
                tokens.addElement(String.valueOf((char)t));
                break;
            }
            t = st.nextToken();
        } while (t != StreamTokenizer.TT_EOF);
        resize(300, 300);
        show();
    }

    int line = 0;
    void paintLine(Graphics g, Font f, int start, int end, 
                   int y, int ht, int ac) {
        int x = 0;
        FontMetrics fm = g.getFontMetrics(f);

        if (line++ % 2 == 0) {
            g.setColor(Color.black);
            g.fillRect(0, y, size().width, ht);
            g.setColor(Color.white);
        } else {
            g.setColor(Color.white);
            g.fillRect(0, y, size().width, ht);
            g.setColor(Color.black);
        }
        g.setFont(f);
        for (int i=start; i<end; i++) {
            Object tk = tokens.elementAt(i);
            if (tk instanceof Font) {
                g.setFont(f = (Font)tk);
                fm = g.getFontMetrics();
            } else {
                g.drawString((String)tk, x, y + ac);
                x += fm.stringWidth((String)tk) + fm.charWidth(' ');
            }
        }
    }

    public void paint(Graphics g) {
        Insets insets = insets();
        int w = size().width-insets.left-insets.right;
        Font f = defaultFont;
        Font startFont = f;
        FontMetrics fm = g.getFontMetrics(f);

        g.clearRect(0, 0, w, size().height);
        g.setFont(f);
        
        int start = 0, x = 0, y = 0;
        int ht = fm.getMaxAscent() + fm.getLeading();
        int base = fm.getMaxAscent();
        line = 0;
        for (int i=0; i<tokens.size(); i++) {
            Object token = tokens.elementAt(i);
            if (token instanceof Font) {
                f = (Font)token;
                g.setFont(f);
                fm = g.getFontMetrics(f);
            } else {
                x += fm.stringWidth((String)token);
                if (x > w) {
                    paintLine(g, startFont, start, i, y, ht, base);
                    startFont = f;
                    start = i--;
                    y += ht;
                    x = ht = base = 0;
                } else {
                    if (y == 0) {
                        ht = Math.max(ht, fm.getMaxAscent() + fm.getDescent());
                        base = Math.max(base, fm.getMaxAscent());
                    } else {
                        ht = Math.max(ht, fm.getHeight());
                        base = Math.max(base, fm.getAscent()+fm.getLeading());
                    }
                    x += fm.charWidth(' ');
                }
            }
        }
        paintLine(g, startFont, start, tokens.size(), y, ht, base);
    }

    static public void main(String[] args) {
        try {
            new Main(args[0]);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
