import java.awt.*;
class Main extends Frame {
    final static int ITEMS = 10;
    List ltList = new List(ITEMS, true);
    List rtList = new List(0, true);

    Main() {
        super("List Example");
        GridBagLayout gbl = new GridBagLayout();

        setLayout(gbl);
        add(ltList, 0, 0, 1, 5, 1.0, 1.0);
        add(rtList, 2, 0, 1, 5, 1.0, 1.0);
        add(new Button(">"), 1, 0, 1, 1, 0, 1.0);
        add(new Button(">>"), 1, 1, 1, 1, 0, 1.0);
        add(new Button("<"), 1, 2, 1, 1, 0, 1.0);
        add(new Button("<<"), 1, 3, 1, 1, 0, 1.0);
        add(new Button("!"), 1, 4, 1, 1, 0, 1.0);

        for (int i=0; i<ITEMS; i++) {
           ltList.addItem("item "+i);
        }
        pack();
        show();
    }

    void add(Component comp,
            int x, int y, int w, int h, double weightx, double weighty) {
        GridBagLayout gbl = (GridBagLayout)getLayout();
        GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.BOTH;
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = w;
        c.gridheight = h;
        c.weightx = weightx;
        c.weighty = weighty;
        add(comp);
        gbl.setConstraints(comp, c);
    }

    void reverseSelections(List l) {
        for (int i=0; i<l.countItems(); i++) {
            if (l.isSelected(i)) {
                l.deselect(i);
            } else {
                l.select(i);
            }
        }
    }

    void deselectAll(List l) {
        for (int i=0; i<l.countItems(); i++) {
            l.deselect(i);
        }
    }

    void replaceItem(List l, String item) {
        for (int i=0; i<l.countItems(); i++) {
            if (l.getItem(i).equals(item)) {
                l.replaceItem(item + "*", i);
            }
        }
    }

    void move(List l1, List l2, boolean all) {
        if (all) {
            for (int i=0; i<l1.countItems(); i++) {
                l2.addItem(l1.getItem(i));
            }
            l1.delItems(0, l1.countItems()-1);    // or l1.clear();
        } else {
            String[] items = l1.getSelectedItems();
            int[] itemIndexes = l1.getSelectedIndexes();

            deselectAll(l2);
            for (int i=0; i<items.length; i++) {
                l2.addItem(items[i]);        // add it
                l2.select(l2.countItems()-1);// and select it
                if (i == 0) {
                    l2.makeVisible(l2.countItems()-1);
                }
            }
            for (int i=itemIndexes.length-1; i>=0; i--) {
                l1.delItem(itemIndexes[i]);
            }
        }
    }

    public boolean action(Event evt, Object arg) {
        if (">".equals(arg)) {
            move(ltList, rtList, false);
        } else if (">>".equals(arg)) {
            move(ltList, rtList, true);
        } else if ("<".equals(arg)) {
            move(rtList, ltList, false);
        } else if ("<<".equals(arg)) {
            move(rtList, ltList, true);
        } else if ("!".equals(arg)) {
            if (ltList.getSelectedItems().length > 0) {
                reverseSelections(ltList);
            } else if (rtList.getSelectedItems().length > 0) {
                reverseSelections(rtList);
            }
        } else if (evt.target == rtList || evt.target == ltList) {
            replaceItem((List)evt.target, (String)arg);
        } else {
            return false;
        }
        return true;
    }

    public boolean handleEvent(Event evt) {
        if (evt.id == Event.LIST_SELECT
               || evt.id == Event.LIST_DESELECT) {
            if (evt.target == ltList) {
                deselectAll(rtList);
            } else if (evt.target == rtList) {
                deselectAll(ltList);
            }
            return true;
        }
        return super.handleEvent(evt);
    }

    public static void main(String[] args) {
        new Main();
    }
}
