import java.awt.*;
import java.util.*;
public class Main extends Frame implements MenuContainer {
    Font font = new Font("Courier", Font.PLAIN, 14);
    FontMetrics fontM;

    // Contains all the menu in this container.
    Vector menus = new Vector();

    // Contains all the menu items in this container.
    Vector menuItems = new Vector();

    Main(Vector menus) {
        super("MenuContainer Example");
        this.menus = menus;
        Vector labels = getLabels();
        int width = 0;

        // Determine how large the frame should be to show all the labels.
        fontM = getFontMetrics(font);
        for (int i=0; i<labels.size(); i++) {
            width = Math.max(width, fontM.stringWidth((String)labels.elementAt(i)));
        }
        resize(width, labels.size() * fontM.getHeight());
        show();
    }

    public Font getFont() {
        return font;
    }

    // Generates a vector of labels.  The menu item labels are indented.
    Vector getLabels() {
        Vector result = new Vector();
        int width = 0;
        int i;

        for (i=0; i<menus.size(); i++) {
            Menu menu = (Menu)menus.elementAt(i);
            result.addElement(menu.getLabel());
            menuItems.addElement(null);
            for (int j=0; j<menu.countItems(); j++) {
                result.addElement("  "+menu.getItem(j).getLabel());
                menuItems.addElement(menu.getItem(j));
            }
        }
        return result;
    }
        
    // Paints the labels.
    public void paint(Graphics g) {
        Vector labels = getLabels();
        int y = fontM.getAscent();

        g.setFont(font);
        for (int i=0; i<labels.size(); i++) {
            String label = (String)labels.elementAt(i);
    
            // Paint menu label blue
            if (label.charAt(0) != ' ') {
                g.setColor(Color.blue);
            }
            g.drawString(label, 0, y);
            y += fontM.getHeight();
            g.setColor(Color.black);
        }
    }

    // Determines which item was clicked on and generates an event for that item.
    public boolean mouseDown(Event evt, int x, int y) {
        Vector labels = getLabels();
        int item = y / fontM.getHeight();
        
        // Don't generate events when the click it not on a label.
        if (item < labels.size()) {
            String label = ((String)labels.elementAt(item));

            //Don't generate event for menu labels.
            if (label.charAt(0) == ' ') {
                evt = new Event(menuItems.elementAt(item), 
                                Event.ACTION_EVENT, label.trim());
                postEvent(evt);
                System.out.println(evt);
                return true;
            }
        }
        return false;
    }

    public void remove(MenuComponent mcomp) {
        menus.removeElement(mcomp);
    }

    static public void main(String[] args) {
        // Now create a few menus.
        Vector menus = new Vector();
        Menu menu = new Menu("File");

        menu.add("New");
        menu.add("Open...");
        menus.addElement(menu);
        menu = new Menu("Edit");
        menu.add("Cut");
        menu.add("Paste");
        menus.addElement(menu);
        new Main(menus);
    }
}
